@@ -1,7 +1,7 @@
 package Digest::CRC;
 
 use strict;
-use vars qw($VERSION @ISA @EXPORT_OK %_typedef);
+use vars qw($VERSION $XS_VERSION @ISA @EXPORT_OK %_typedef);
 
 require Exporter;
 
@@ -16,50 +16,32 @@ require Exporter;
  crc32_hex crc32_base64
 );
 
-$VERSION = '0.08';
+$VERSION    = '0.11';
+$XS_VERSION = $VERSION;
+$VERSION    = eval $VERSION;
 
-%_typedef = (
-# name,  [width,init,xorout,refout,poly,refin);
-  crc8 => [8,0,0,0,0x07,0],
-  crcccitt => [16,0xffff,0,0,0x1021,0],
-  crc16 => [16,0,0,1,0x8005,1],
-  crc32 => [32,0xffffffff,0xffffffff,1,0x04C11DB7,1],
-);
+eval {
+  # PERL_DL_NONLAZY must be false, or any errors in loading will just
+  # cause the perl code to be tested
+  local $ENV{PERL_DL_NONLAZY} = 0 if $ENV{PERL_DL_NONLAZY};
+  require DynaLoader;
+  local @ISA = qw(DynaLoader);
+  bootstrap Digest::CRC $XS_VERSION;
+  1
+};
 
-sub new {
-  my $that=shift;
-  my %params=@_;
-  my $class = ref($that) || $that;
-  my $self = {map { ($_ => $params{$_}) }
-                      qw(type width init xorout poly refin refout)};
-  bless $self, $class;
-  $self->reset();
-  $self
+sub _reflectperl {
+  my ($in, $width) = @_;
+  my $out = 0;
+  for(my $i=1; $i < ($width+1); $i++) {
+    $out |= 1 << ($width-$i) if ($in & 1);
+    $in=$in>>1;
+  }
+  $out;
 }
 
-sub reset {
-  my $self = shift;
-  my $typeparams;
-  # default is crc32 if no type and no width is defined
-  if (!defined($self->{type}) && !defined($self->{width})) {
-    $self->{type} = "crc32";
-  }
-  if (defined($self->{type}) && exists($_typedef{$self->{type}})) {
-    $typeparams = $_typedef{$self->{type}};
-    $self->{width} = $typeparams->[0],
-    $self->{init} = $typeparams->[1],
-    $self->{xorout} = $typeparams->[2],
-    $self->{refout} = $typeparams->[3],
-    $self->{poly} = $typeparams->[4],
-    $self->{refin} = $typeparams->[5],
-  }
-  $self->{_tab} = [_tabinit($self->{width}, $self->{poly}, $self->{refin})];
-  delete $self->{_data};
-  $self
-}
-
-#########################################
-# Private init functions:
+# Only load the non-XS stuff on demand
+defined &_crc or eval <<'ENOXS';
 
 sub _reflect {
   my ($in, $width) = @_;
@@ -94,18 +76,20 @@ sub _tabinit {
 	}
       }
     }
-    push @crctab, $r&2**$width-1;
+    my $x=$r&2**$width-1;
+    push @crctab, $x;
   }
-  @crctab;
+  \@crctab;
 }
 
 sub _crc {
   my ($message,$width,$init,$xorout,$refin,$refout,$tab) = @_;
   my $crc = $init;
-  $crc = _reflect($crc,$width) if $refin;
+  $crc = __reflect($crc,$width) if $refin;
   my $pos = -length $message;
   my $mask = 2**$width-1;
   while ($pos) {
+  prinddt "pos=$pos\n";
     if ($refin) {
       $crc = ($crc>>8)^$tab->[($crc^ord(substr($message, $pos++, 1)))&0xff]
     } else {
@@ -121,6 +105,50 @@ sub _crc {
   $crc & $mask;
 }
 
+ENOXS
+
+%_typedef = (
+# name,  [width,init,xorout,refout,poly,refin);
+  crc8 => [8,0,0,0,0x07,0],
+  crcccitt => [16,0xffff,0,0,0x1021,0],
+  crc16 => [16,0,0,1,0x8005,1],
+  crc32 => [32,0xffffffff,0xffffffff,1,0x04C11DB7,1],
+);
+
+sub new {
+  my $that=shift;
+  my %params=@_;
+  my $class = ref($that) || $that;
+  my $self = {map { ($_ => $params{$_}) }
+                      qw(type width init xorout poly refin refout cont)};
+  bless $self, $class;
+  $self->reset();
+  map { if (defined($params{$_})) { $self->{$_} = $params{$_} } }
+                      qw(type width init xorout poly refin refout cont);
+  $self
+}
+
+sub reset {
+  my $self = shift;
+  my $typeparams;
+  # default is crc32 if no type and no width is defined
+  if (!defined($self->{type}) && !defined($self->{width})) {
+    $self->{type} = "crc32";
+  }
+  if (defined($self->{type}) && exists($_typedef{$self->{type}})) {
+    $typeparams = $_typedef{$self->{type}};
+    $self->{width} = $typeparams->[0],
+    $self->{init} = $typeparams->[1],
+    $self->{xorout} = $typeparams->[2],
+    $self->{refout} = $typeparams->[3],
+    $self->{poly} = $typeparams->[4],
+    $self->{refin} = $typeparams->[5],
+  }
+  $self->{_tab} = _tabinit($self->{width}, $self->{poly}, $self->{refin});
+  $self->{_data} = undef;
+  $self
+}
+
 #########################################
 # Private output converter functions:
 sub _encode_hex { sprintf "%x", $_[0] }
@@ -150,10 +178,18 @@ sub addfile {
     require Symbol;
     $fh = Symbol::qualify($fh, scalar caller);
   }
-  # $self->{_data} .= do{local$/;<$fh>};
   my $read = 0;
   my $buffer = '';
-  $self->add($buffer) while $read = read $fh, $buffer, 8192;
+  my $crc;
+  my $oldinit = $self->{init};
+  while ($read = read $fh, $buffer, 32*1024) {
+    $self->add($buffer);
+    $crc = $self->digest;
+    $self->{cont}=1;
+    $self->{init}=$crc;
+  }
+  $self->{init} = $oldinit;
+  $self->{_crc} = $crc;
   die __PACKAGE__, " read failed: $!" unless defined $read;
   $self
 }
@@ -163,8 +199,21 @@ sub add_bits {
 
 sub digest {
   my $self = shift;
-  _crc($self->{_data},$self->{width},$self->{init},$self->{xorout},
-       $self->{refin},$self->{refout},$self->{_tab});
+  my $crc;
+  if (!$self->{_crc}) {
+    my $init = $self->{init};
+    if ($self->{cont}) {
+      $init = ($self->{init} ^ $self->{xorout});
+      $init = _reflect($init, $self->{width}) if $self->{refin};
+    }
+    $crc =_crc($self->{_data},$self->{width},$init,$self->{xorout},
+	 $self->{refin},$self->{refout},$self->{_tab});
+  } else {
+    $crc = $self->{_crc};
+    $self->{_crc} = undef;
+  }
+  $self->{_data} = undef;
+  $crc
 }
 
 sub hexdigest {
@@ -194,7 +243,7 @@ sub clone {
 
 sub crc {
   my ($message,$width,$init,$xorout,$refout,$poly,$refin) = @_;
-  _crc($message,$width,$init,$xorout,$refin,$refout,[_tabinit($width,$poly,$refin)]);
+  _crc($message,$width,$init,$xorout,$refin,$refout,_tabinit($width,$poly,$refin));
 }
 
 # CRC8
@@ -262,8 +311,8 @@ Digest::CRC - Generic CRC functions
   use Digest::CRC;
 
   $ctx = Digest::CRC->new(type=>"crc16");
-  $ctx = Digest::CRC->new(width=>16, init=>0x0000, xorout=>0x0000, 
-                          poly=>0x8005, refin=>1, refout=>1);
+  $ctx = Digest::CRC->new(width=>16, init=>0x2345, xorout=>0x0000, 
+                          poly=>0x8005, refin=>1, refout=>1, cont=>1);
 
   $ctx->add($data);
   $ctx->addfile(*FILE);
